#include <windows.h>
#include <d3d9.h>
#include <d3dx9.h>

// 

//  
HWND               hWnd       = NULL;

//   Direct3D
IDirect3D9*        pD3D       = NULL;

//  
IDirect3DDevice9*  pDevice = NULL;

//   
IDirect3DVertexBuffer9* vb = NULL;

//  
IDirect3DTexture9* tex = NULL;

//,   
struct Vertex
{
	float x, y, z;
	float u, v;

	static const DWORD FVF;
};

const DWORD Vertex::FVF = D3DFVF_XYZ | D3DFVF_TEX1;

//   
Vertex Vertices[] =
{
	{-1.0f, 1.0f,-1.0f, 0.0f, 0.0f}, // 0
	{ 1.0f, 1.0f,-1.0f, 1.0f, 0.0f}, // 1
	{-1.0f,-1.0f,-1.0f, 0.0f, 1.0f}, // 2
	{ 1.0f,-1.0f,-1.0f, 1.0f, 1.0f}, // 3
};


//  
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, 
				   LPSTR lpCmdLine, int nCmdShow);
LRESULT CALLBACK WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
void Init(void);
void Shutdown(void);
void Render(void);


//

//  
int WINAPI WinMain( HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPSTR     lpCmdLine,
					int       nCmdShow)
{
	// 
	WNDCLASS wc;

	// 
	MSG        Msg;
    
	//   
	wc.style         = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc   = WndProc;
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = 0;
    wc.hInstance     = hInstance;
    wc.hIcon         = LoadIcon(0, IDI_APPLICATION);
    wc.hCursor       = LoadCursor(0, IDC_ARROW);
    wc.hbrBackground = static_cast<HBRUSH>(GetStockObject(BLACK_BRUSH));
    wc.lpszMenuName  = 0;
    wc.lpszClassName = "Plain";

	//    - 
	if(!RegisterClass(&wc))
	{
		MessageBox(0, "   .", 0, 0);
        exit(1);
	}
	
	//  
	hWnd = CreateWindowEx(WS_EX_OVERLAPPEDWINDOW,
						  "Plain", 
                          "Direct3D (DX9) - Plain",
						  WS_OVERLAPPEDWINDOW,
					      0, 0, 640, 480, NULL, NULL, hInstance, NULL );

	//    - 
	if(hWnd == NULL)
	{
		MessageBox(0, "  ", 0, 0);
        exit(1);
	}
		
	//   
    ShowWindow(hWnd, nCmdShow);
    UpdateWindow(hWnd);

	//  
	ZeroMemory(&Msg, sizeof(Msg));
 
	// Direct3D
	Init();

	//      
	while(Msg.message != WM_QUIT)
	{
		//   ,
		if(PeekMessage(&Msg, NULL, 0, 0, PM_REMOVE))
		{ 
			//    
			TranslateMessage(&Msg);
			DispatchMessage(&Msg);
		}

		//
		Render();
	}

	// 
	Shutdown();

	//  
    UnregisterClass("Plain", wc.hInstance);

	return Msg.wParam;
}

//  ,   
// 
LRESULT CALLBACK WndProc(HWND   hWnd, 
						 UINT   msg, 
						 WPARAM wParam, 
						 LPARAM lParam)
{
	static POINT LastMPos;
	static POINT CurrMPos;
	static bool lButton;

	switch(msg)
	{
	case WM_KEYDOWN:
		{
			switch(wParam)
			{
			case VK_ESCAPE:
				PostQuitMessage(0);
				break;
			}
		}
		break;
		
        case WM_DESTROY:
		{
            PostQuitMessage(0);
		}
        break;

		default:
		{
			return DefWindowProc( hWnd, msg, wParam, lParam );
		}
		break;
	}

	return 0;
}

//  Direct3D,  
//  ,    
void Init(void)
{
	//CAPS 
	D3DCAPS9 caps;
	
	//   
	pD3D = Direct3DCreate9(D3D_SDK_VERSION);
	
	pD3D->GetDeviceCaps(D3DADAPTER_DEFAULT,	D3DDEVTYPE_HAL,	&caps);	
	
	//   
	int vp = 0;
	if(caps.DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT)
		vp = D3DCREATE_HARDWARE_VERTEXPROCESSING;
	else
		vp = D3DCREATE_SOFTWARE_VERTEXPROCESSING;

	//   .
	D3DPRESENT_PARAMETERS d3dpp = {0};
	d3dpp.BackBufferWidth            = 640;
	d3dpp.BackBufferHeight           = 480;
	d3dpp.BackBufferFormat           = D3DFMT_A8R8G8B8; // 
	d3dpp.BackBufferCount            = 1;
	d3dpp.MultiSampleType            = D3DMULTISAMPLE_NONE;
	d3dpp.MultiSampleQuality         = 0;
	d3dpp.SwapEffect                 = D3DSWAPEFFECT_DISCARD;
	d3dpp.hDeviceWindow              = hWnd;
	d3dpp.Windowed                   = true; 
	d3dpp.EnableAutoDepthStencil     = true;
	d3dpp.AutoDepthStencilFormat     = D3DFMT_D24S8; //   
	d3dpp.Flags                      = 0;
	d3dpp.FullScreen_RefreshRateInHz = D3DPRESENT_RATE_DEFAULT;
	d3dpp.PresentationInterval       = D3DPRESENT_INTERVAL_IMMEDIATE;

	//  
	pD3D->CreateDevice(D3DADAPTER_DEFAULT, 
					   D3DDEVTYPE_HAL, 
					   hWnd,
                       D3DCREATE_HARDWARE_VERTEXPROCESSING,
                       &d3dpp, 
					   &pDevice);
	
	// 
	pDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	// Z - 
	pDevice->SetRenderState(D3DRS_ZENABLE, TRUE);

	//  
	D3DXMATRIX matProj;
	D3DXMatrixPerspectiveFovLH(&matProj, D3DXToRadian(45.0f), 
                                640.0f / 480.0f, 0.1f, 100.0f);
	pDevice->SetTransform( D3DTS_PROJECTION, &matProj );


	//  
	pDevice->CreateVertexBuffer(4*sizeof(Vertex),
		                        D3DUSAGE_WRITEONLY,
								Vertex::FVF,
		                        D3DPOOL_DEFAULT,
								&vb,
								NULL);
	WORD *pVertices = NULL;

	// 
	vb->Lock(0, sizeof(Vertices), (void**)&pVertices, 0);
	//     
	memcpy(pVertices, Vertices, sizeof(Vertices));
	// 
	vb->Unlock();

	//   
	D3DXCreateTextureFromFile(pDevice, "mud.jpg", &tex);
	
	//  
	pDevice->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	pDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
}

//  . 
//     ,
//  ""
void Shutdown(void)
{

	if(vb != NULL)
		vb->Release(); 

    if(pDevice != NULL)
        pDevice->Release();

    if(pD3D != NULL)
        pD3D->Release();
}

//  
void Render(void)
{
	//   Z -      
    pDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,
                         D3DCOLOR_COLORVALUE(0.0f,0.0f,0.0f,1.0f), 1.0f, 0);

	// 
	D3DXMATRIX matWorld;

	//  
	D3DXMatrixTranslation(&matWorld, 0.0f, 0.0f, 5.0f);

	pDevice->SetTransform(D3DTS_WORLD, &matWorld);

	// 
    pDevice->BeginScene();

	//  
	pDevice->SetTexture(0, tex);

	//   
	pDevice->SetStreamSource(0, vb, 0, sizeof(Vertex));
	//   
	pDevice->SetFVF(Vertex::FVF);

	// 
    pDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);

	// 
    pDevice->EndScene();
	//  
    pDevice->Present(NULL, NULL, NULL, NULL);
}
